import java.util.ArrayList;

public class Container extends Thing{
    private ArrayList<Thing> contents; //Stores the Things in the container

    /*
    * Creates a Thing that can hold other Things.
    */
    public Container(String[] names, String description, boolean movable, boolean hidden){
        super(names, description, movable, hidden);
        contents = new ArrayList<Thing>();
    }

    /* Returns the container's contents */
    @Override
    public ArrayList<Thing> getContents(){
        return contents;
    }

    /* Searches for and returns the index of a Thing from 'contents' based on the name passed in. Returns -1 if none exists. */
    @Override
    public int findIndexOfThing(String name){
        for(int i = 0; i < contents.size(); i++){
            for(int j = 0; j < contents.get(i).getAllNames().length; j++){
                if(contents.get(i).getAllNames()[j].toLowerCase().equals(name)){
                    return i;
                }
            }
        }
        return -1;
    }

    /* Searches for and returns a Thing from 'contents' based on the name passed in. Returns null if none exists. */
    @Override
    public Thing findThing(String name){
        int index = findIndexOfThing(name);
        return contents.get(index);
    }

    /* Adds a Thing to 'contents'. */
    @Override
    public void addThing(Thing thing){
        contents.add(thing);
    }

    /* Removes and returns a Thing from 'contents'. */
    @Override
    public Thing removeThing(String name){
        int index = findIndexOfThing(name);
        return contents.remove(index);
    }

    /* Returns "It has [list of nonhidden contents] in it." If there are none, returns "nothing" in place of the list. */
    @Override
    public String getFormattedContents(){
        int numThings = 0;
        String result = "There's";
        //Tests if 'contents' has objects in it
        if(contents.size() != 0){
            for(int i = 0; i < contents.size(); i++){
                //Tests whether each object is hidden before including it in the String
                if(!contents.get(i).isHidden()){
                    //Tests whether or not the current Thing is the last nonhidden Thing in 'contents'
                    boolean last = true;
                    for(int j = i + 1; j < contents.size(); j++){
                        if(!contents.get(j).isHidden()){
                            last = false;
                            break;
                        }
                    }
                    //Adds a comma before the name if there has already been at least one nonhidden Thing and there is more than two nonhidden Things total
                    if(numThings >=1 && !(numThings == 1 && last)){
                        result += ",";
                    }
                    //Adds an "and" before the last nonhidden Thing in 'contents'
                    if(numThings >= 1 && last){
                        result += " and";
                    }
                    numThings += 1;
                    String name = contents.get(i).getDefaultName();
                    //Adds the appropriate article and then the name of the Thing to the resulting String
                    result += " a";
                    if(name.charAt(0) == 'a' || name.charAt(0) == 'e' ||name.charAt(0) == 'i' || name.charAt(0) == 'o' || name.charAt(0) == 'u'){
                        result += "n";
                    }
                    result += " " + name;
                }
            }
        }
        //If 'contents' had no nonhidden objects, print that the container has nothing in it
        if(numThings == 0){
            result += " nothing";
        }
        result += " in it.";
        return result;
    }

    /* Prints the formatted name, description, and list of contents. */
    @Override
    public String toString(){
        return super.toString() + " " + getFormattedContents();
    }
}