public class Door extends Thing{
    private boolean open; //Stores whether or not the Door is open
    private Room otherSide; //Stores the Room on the other side of the Door
    private String direction; //Stores the direction in which the Door leads

    /* Creates an openable Thing with a direction and target room. */
    public Door(String[] names, String description, Room otherSide, String direction){
        super(names, description, false, true);
        this.otherSide = otherSide;
        this.direction = direction;
    }

    /* Returns the direction. */
    @Override
    public String getDirection(){
        return direction;
    }

    /* Returns the room on the "other side" of the door. */
    @Override
    public Room getOtherSide(){
        return otherSide;
    }

    /* Returns the state passed in if it exists, otherwise returns false. */
    @Override
    public boolean isState(String state){
        if(state.equals("open")){
            return open;
        }
        return super.isState(state);
    }

    /*
    * Changes the open state and all inherited states.
    * Returns a String describing the action that happened.
    */
    @Override
    public String changeState(String command){
        if(command.equals("open")){
            if(open){
                return "It's already open!";
            }
            open = true;
            return "You open it.";
        }
        if(command.equals("close")){
            if(!open){
                return "It's already closed!";
            }
            open = false;
            return "You close it.";
        }
        return super.changeState(command);
    }

    /* Returns the name, the direction in which the door leads, and whether or not the door is closed. */
    @Override
    public String toString(){
        String result = super.toString() + " It leads " + direction + ". It's ";
        if(open){
            result += "open.";
        }
        else{
            result += "closed.";
        }
        return result;
    }
}