public class FueledMachine extends Machine{
    private boolean hasFuel; //Stores whether or not the Machine is fueled
    private String[] fuels; //Stores the accepted Fuels that can be used on the Machine

    /* Creates a Machine that requires another Thing to be turned on. */
    public FueledMachine(String[] names, String[] fuels, String description, String startText, String finishText, boolean lamp, boolean movable, boolean hidden){
        super(names, description, startText, finishText, lamp, movable, hidden);
        this.fuels = fuels;
    }

    /* Returns the state passed in if it exists, otherwise returns false. */
    @Override
    public boolean isState(String state){
        if(state.equals("fueled")){
            return hasFuel;
        }
        if(state.equals("fuelable")){
            return true;
        }
        return super.isState(state);
    }

    /*
    * Changes the fueled state if a compatible fuel is passed in, as well as all inherited states.
    * Returns a String describing the action that happened.
    */
    @Override
    public String changeState(String command){
        for(int i = 0; i < fuels.length; i++){
            if(command.equals(fuels[i])){
                if(hasFuel){
                    return "You already did that!";
                }
                hasFuel = true;
                return super.changeState("on");
            }
        }
        if(command.equals("on") && !hasFuel){
            return "Nothing happens.";
        }
        return super.changeState(command);
    }
}