public class LockableDoor extends Door{
    boolean locked; //Stores whether or not the Door is locked
    String key; //Stores the name of the Thing that can be used to unlock the door
    
    /* Creates a Door that can be locked and unlocked with a specific key. */
    public LockableDoor(String[] names, String description, Room otherSide, String direction, boolean locked, String key){
        super(names, description, otherSide, direction);
        this.locked = locked;
        this.key = key;
    }

    /* Returns the state passed in if it exists, otherwise returns false. */
    @Override
    public boolean isState(String state){
        if(state.equals("lockable")){
            return true;
        }
        if(state.equals("locked")){
            return locked;
        }
        return super.isState(state);
    }

    /*
    * Changes the locked state by passing in a compatible key, as well as all inherited states.
    * Returns a String describing the action that happened.
    */
    @Override
    public String changeState(String command){
        if(command.equals(key)){
            if(locked){
                locked = false;
                return "You unlock it.";
            }
            else{
                locked = true;
                return "You lock it.";
            }
        }
        return super.changeState(command);
    }

    /* Prints out the name, the direction in which the door leads, and whether or not the door is closed and locked. */
    @Override
    public String toString(){
        String result = super.toString();
        if(!super.isState("open") && locked){
            result = result.substring(0, result.length() -1) + " and locked.";
        }
        return result;
    }
}