public class Machine extends Thing{
    private boolean on; //Stores whether or not the Machine is on
    private boolean lamp; //Stores whether or not the Machine is a lamp and can light up a room
    private String startText; //Stores the text to be printed when the machine is turned on
    private String finishText; //Stores the text to be printed when the machine is turned off

    /* Creates a Thing that can be turned on and off. */
    public Machine(String[] names, String description, String startText, String finishText, boolean lamp, boolean movable, boolean hidden){
        super(names, description, movable, hidden);
        this.startText = startText;
        this.finishText = finishText;
        this.lamp = lamp;
    }

    /* Returns the state passed in if it exists, otherwise returns false. */
    @Override
    public boolean isState(String state){
        if(state.equals("on")){
            return on;
        }
        if(state.equals("lamp")){
            return lamp;
        }
        return super.isState(state);
    }

    /*
    * Changes the on/off state and all inherited states.
    * Returns a String describing the action that happened.
    */
    @Override
    public String changeState(String command){
        if(command.equals("on")){
            if(on){
                return "It's already on!";
            }
            on = true;
            return startText;
        }
        else if(command.equals("off")){
            if(!on){
                return "It's already off!";
            }
            on = false;
            return finishText;
        }
        //Allows the method to change any inherited states if the result is neither "on" nor "off". Prints the default response if no corresponding states exist.
        return super.changeState(command);
    }
}