import java.util.ArrayList;

public class Player extends Thing{
    private ArrayList<Thing> inventory; //A list of Things currently held by the Player.

    /*
    * Provides an implementation of the player that takes no arguments.
    * Allows the player to exist in rooms and hold objects.
    * Only one Player object should be created.
    */
    public Player(){
        super(new String[]{"player", "self", "myself"}, "You look like yourself.", false, true);
        inventory = new ArrayList<Thing>();
    }
    
    /* Returns the Player's inventory */
    @Override
    public ArrayList<Thing> getContents(){
        return inventory;
    }

    /* Searches for and returns the index of a Thing from 'inventory' based on the name passed in. Returns -1 if none exists. */
    @Override
    public int findIndexOfThing(String name){
        for(int i = 0; i < inventory.size(); i++){
            for(int j = 0; j < inventory.get(i).getAllNames().length; j++){
                if(inventory.get(i).getAllNames()[j].toLowerCase().equals(name)){
                    return i;
                }
            }
        }
        return -1;
    }

    /* Searches for and returns a Thing from 'inventory' based on the name passed in. Returns null if none exists. */
    @Override
    public Thing findThing(String name){
        int index = findIndexOfThing(name);
        if(index != -1){
            return inventory.get(index);
        }
        return null;
    }

    /* Adds a Thing to the Player's 'inventory' in alphabetical order. */
    @Override
    public void addThing(Thing thing){
        boolean added = false;
        if(inventory.size() != 0){
            for(int i = 0; i < inventory.size(); i++){
                if(thing.getDefaultName().compareTo(inventory.get(i).getDefaultName()) < 0){
                    inventory.add(i, thing);
                    added = true;
                    break;
                }
            }
        }
        if(!added){
            inventory.add(thing);
        }
        thing.setKnown();
    }

    /* Removes a Thing from the Player's 'inventory'. */
    @Override
    public Thing removeThing(String thing){
        int index = findIndexOfThing(thing);
        return inventory.remove(index);
    }

    /* Overrides the default unhide behavior to prevent the player from being unhidden. */
    @Override
    public String changeState(String command){
        if(command.equals("unhidden")){
            return "";
        }
        return super.changeState(command);
    }

    /* Returns "You're carrying [list of nonhidden contents]." If there are none, returns "nothing" in place of the list.*/
    @Override
    public String getFormattedContents(){
        int numThings = 0;
        String result = "You're carrying";
        //Tests if 'inventory' has objects in it
        if(inventory.size() != 0){
            for(int i = 0; i < inventory.size(); i++){
                //Tests whether each object is hidden before including it in the String
                if(!inventory.get(i).isHidden()){
                    //Tests whether or not the current Thing is the last nonhidden Thing in 'inventory'
                    boolean last = true;
                    for(int j = i + 1; j < inventory.size(); j++){
                        if(!inventory.get(j).isHidden()){
                            last = false;
                            break;
                        }
                    }
                    //Adds a comma before the name if there has already been at least one nonhidden Thing and there is more than two nonhidden Things total
                    if(numThings >=1 && !(numThings == 1 && last)){
                        result += ",";
                    }
                    //Adds an "and" before the last nonhidden Thing in 'inventory'
                    if(numThings >= 1 && last){
                        result += " and";
                    }
                    numThings += 1;
                    String name = inventory.get(i).getDefaultName();
                    //Adds the appropriate article and then the name of the Thing to the resulting String
                    result += " a";
                    if(name.charAt(0) == 'a' || name.charAt(0) == 'e' ||name.charAt(0) == 'i' || name.charAt(0) == 'o' || name.charAt(0) == 'u'){
                        result += "n";
                    }
                    result += " " + name;
                }
            }
        }
        //If 'inventory' had no nonhidden objects, print that the player is carrying nothing.
        if(numThings == 0){
            result += " nothing";
        }
        result += ".";
        return result;
    }

    /* Prints just the description instead of formatting along with the default name. */
    @Override
    public String toString(){
        return getDescription();
    }
}