import java.util.Scanner;

public class World{

    /*
    * The World class exists to hold the environment and run the parser.
    * It also determines the current location of the player at each turn.
    */
    public static void main(String[] args){
        String name = "Escape";
        String author = "Peter Marquill";
        String version = "1.1";
        String date = "April 2024";
        Room[] rooms = createEnvironment();
        Parser parser = new Parser(rooms);
        Scanner parserInput = new Scanner(System.in);
        
        //Prints the game information and introduction
        System.out.println();
        System.out.println("You were just hoping for an nice drive in the countryside. Nothing more. You would even have settled for an okay drive in the countryside. But here you are, sitting in a car whose steering has suddenly been taken entirely from your control and which is now speeding down a dusty, bumpy dirt road through fields of wheat.");
        System.out.println();
        System.out.println("The car stops as suddenly as it started, and you're thrown forward. When you sit back up, you realize you've pulled to a stop in front of a quaint farmhouse, complete with peeling white paint and a brand-new front porch. An invisible force pushes you roughly out the car door and up the steps.");
        System.out.println();
        System.out.println("As you stumble inside, you hear the heavy door swing close behind you with a bang, and the sound of the lock turning...");
        System.out.println();
        System.out.println("**********");
        System.out.println();
        System.out.println("Welcome to " + name + "!");
        System.out.println("This is a traditional text adventure based on early games like Colossal Cave Adventure.");
        System.out.println("For a list of possible commands and more information on how to use them, please type \"help\".");
        System.out.println();
        System.out.println(name + " is written by " + author + " and was originally released June 2023. You are playing version " + version + ", released " + date + ".");
        System.out.println();

        //Prints the description of the player's starting room.
        System.out.println(parser.findCurrentRoom(rooms));

        //Boolean values to store whether each special turn-based event has already occured
        Boolean cellarKeyAdded = false;
        Boolean endKeyAdded = false;
        //Takes the player's commands and runs them through the parser.
        while(true){
            System.out.println();
            System.out.print("> ");
            String input = parserInput.nextLine().toLowerCase();
            parser.parse(input);
            //Ends the game when the player exits the house
            if(parser.findCurrentRoom(rooms).getName().equals("The End")){
                break;
            }
            //Implements special rules to be checked every turn
            if(parser.findAnywhere("husky").isState("happy") && !cellarKeyAdded){
                parser.findAnywhere("player").addThing(new Thing(new String[]{"cellar-key", "key"}, "the key to the cellar", true, false));
                System.out.println("\nThe husky looks at you with adoring eyes, before spitting a small key out into your hands.");
                cellarKeyAdded = true;
            }
            if(rooms[5].findIndexOfThing("cake") != -1 && !endKeyAdded){
                rooms[5].addThing(new Thing(new String[]{"iron-key", "iron"}, "large and iron. It looks like it might belong to the front door", true, false));
                endKeyAdded = true;
            }
        }
        System.out.println();
        parserInput.close();
    }

    /* Returns an array of manually-created Rooms. */
    public static Room[] createEnvironment(){
        Room[] rooms = {
            /* 0: Entryway */ new Room("Entryway", "You're in a small mudroom, its cracked flagstone floor mottled with sunlight from the large glass windows. An open doorway leads ahead of you to the north into a short hallway, and behind you, to the south, is the large wooden door to the outdoors."),
            /* 1: Hallway */ new Room("Hallway", "You're in an empty hallway. There are exits to the west[if Library?known]{ (the library)} and east[if Dining Room?known]{ (the dining room)}, and a doorway leads back south to the mudroom. At the end of the hallway is a cramped staircase leading up, and tucked underneath that is a small door leading down to the cellar."),
            /* 2: Cellar */ new LightableRoom("Cellar", "You're in the cellar, full of boxes and barrels. A staircase leads back up to the downstairs hallway."),
            /* 3: Library */ new Room("Library", "You're in a small library full of bookshelves, most of which are empty. You can go back eastward to the hallway[if Crawl Space?known]{ or downward through the trapdoor}."),
            /* 4: Crawl Space */ new LightableRoom("Crawl Space", "You're in a crawl space under the library. It smells like dirt.\n\nThere's a pile of ratty blankets in the corner, and a few long-expired cans of pumpkin soup — it looks like someone might have lived here once?"),
            /* 5: Dining Room */ new Room("Dining Room", "A long table takes up most of the space here. A vase of fresh flowers sits in the center, on top of a linen table runner. You can go north[if Kitchen?known]{ to the kitchen} or west to the hallway."),
            /* 6: Kitchen */ new Room("Kitchen", "You're in a small, country-style kitchen with gingham curtains and wooden cabinets. A doorway leads back into the dining room."),
            /* 7: Upstairs Hallway */ new Room("Upstairs Hallway", "You're in another hallway, almost identical to the first. You can go north[if Blue Bedroom?known]{ to the blue bedroom}, south[if Yellow Bedroom?known]{ to the yellow bedroom}, west[if Billiard Room?known]{ to the billiard room}, or back down the stairs."),
            /* 8: Blue Bedroom */ new Room("Blue Bedroom", "You're in a bedroom with blue wallpaper. A soft bed with a checkered quilt sits facing the southern doorway, a large landscape painting hanging above it."),
            /* 9: Billiard Room */ new Room("Billiard Room", "You're in a small billiard room with panelled walls. The only door leads back east to the hallway."),
            /* 10: Yellow Bedroom */ new Room("Yellow Bedroom", "There's a small, spindly bed in the middle of the room, which is papered in a bright yellow floral print."),
            /* 11: The End */ new Room("The End", "You walk out of the house into the light breeze, taking in the sight of the grassy hills. On the driveway, a car awaits to bring you home.\n\nYou give one last glance behind you at the house, but there's nothing there...")
        };
        
        //Adds items to the Entryway
        rooms[0].addExit("north", rooms[1]);

        rooms[0].addThing(new Player());
        rooms[0].addThing(new Thing(new String[]{"floor", "flagstone"}, "cold beneath your feet", false, true));
        rooms[0].addThing(new Thing(new String[]{"walls", "wall"}, "painted a light cream color", false, true));
        rooms[0].addThing(new Thing(new String[]{"windows", "window", "glass", "sunlight", "tree", "birch", "car", "outside", "hills"}, "bright and clean. Your view is partially obscured by a birch tree, but you can see a car sitting outside, and rolling hills disappearing into the distance", false, true));
        rooms[0].addThing(new Thing(new String[]{"doorway", "hallway"}, "small and opens onto an equally small hallway", false, true));
        rooms[0].addThing(new LockableDoor(new String[]{"wooden-door", "door"}, "large and wooden", rooms[11], "south", true, "iron-key"));

        //Adds items to the Hallway
        rooms[1].addExit("east", rooms[5]);
        rooms[1].addExit("south", rooms[0]);
        rooms[1].addExit("west", rooms[3]);
        rooms[1].addExit("up", rooms[7]);

        rooms[1].addThing(new Thing(new String[]{"floor"}, "nondescript", false, true));
        rooms[1].addThing(new Thing(new String[]{"walls", "wall"}, "painted the same light cream color as the entryway", false, true));
        rooms[1].addThing(new Thing(new String[]{"staircase", "stairs"}, "narrow and rickety, with a wooden bannister", false, true));
        rooms[1].addThing(new Thing(new String[]{"doorway"}, "small. It opens back on to the mudroom", false, true));
        rooms[1].addThing(new LockableDoor(new String[]{"cellar-door", "door", "cellar"}, "small and wooden", rooms[2], "down", true, "cellar-key"));

        //Adds items to the Cellar
        rooms[2].addExit("up", rooms[1]);

        rooms[2].addThing(new Thing(new String[]{"floor"}, "bare, compact dirt", false, true));
        rooms[2].addThing(new Thing(new String[]{"walls", "wall"}, "rough wood", false, true));
        rooms[2].addThing(new Thing(new String[]{"clutter", "barrels", "barrel", "boxes", "box"}, "scattered around the dim cellar. You spot a half-open crate in the corner", false, true));
        rooms[2].addThing(new OpenableContainer(new String[]{"crate"}, "splintery and wooden", false, true));

        rooms[2].findThing("crate").addThing(new Thing(new String[]{"cake-plate", "plate"}, "made of delicate china. You notice a recipe tucked beneath it", false, false));
        rooms[2].findThing("crate").addThing(new Recipe(new String[]{"recipe"}, "cake", new String[]{"cup of flour", "cup of sugar", "egg", "chocolate-bar"}, true));

        //Adds items to the Library
        rooms[3].addExit("east", rooms[1]);

        rooms[3].addThing(new Thing(new String[]{"floor"}, "dark and smooth. As you inspect it, you notice a thin crack — a trapdoor", false, true));
        rooms[3].addThing(new Thing(new String[]{"walls", "wall"}, "made of the same dark wood as the walls", false, true));
        rooms[3].addThing(new LockableDoor(new String[]{"trapdoor", "ring", "keyhole", "door", "crack"}, "made of rough wood with a small metal ring set into it[if trapdoor:locked]{. It has a small, hexagonal keyhole}", rooms[4], "down", true, "hexagonal-cylinder"));
        rooms[3].addThing(new Thing(new String[]{"bookshelves", "bookshelf", "shelves", "shelf"}, "tall and dark. [if book:!known]{This one has a single book on it}[if book:known]{.This one's completely empty}", false, true));
        rooms[3].addThing(new Thing(new String[]{"book", "books"}, "titled \"Adventure Game Tips.\" It reads: \"Stuck? Try examining things more closely. You might find a clue or two\"", true, true));

        //Adds items to the Crawl Space
        rooms[4].addExit("up", rooms[3]);

        rooms[4].addThing(new Thing(new String[]{"floor"}, "bare dirt", false, true));
        rooms[4].addThing(new Thing(new String[]{"walls", "wall"}, "bare dirt", false, true));
        rooms[4].addThing(new Thing(new String[]{"blankets"}, "ratty and full of holes", true, true));
        rooms[4].addThing(new Thing(new String[]{"cans", "can", "soup"}, "strewn about the ground", false, true));
        rooms[4].addThing(new Recipe(new String[]{"paper-scrap", "paper", "scrap"}, "dog-biscuit", new String[]{"peanut butter", "chicken"}, false));

        //Adds items to the Dining Room
        rooms[5].addExit("north", rooms[6]);
        rooms[5].addExit("west", rooms[1]);

        rooms[5].addThing(new Thing(new String[]{"floor","d", "down"}, "dark wood", false, true));
        rooms[5].addThing(new Thing(new String[]{"walls", "wall"}, "a pretty spring green that makes the room feel airy and light", false, true));
        rooms[5].addThing(new Thing(new String[]{"table", "writing", "tabletop"}, "long and dark, surrounded by matching chairs. There's writing carved into the tabletop, reading [if cake->!Dining Room]{\"let there be cake...\"\n\n}[if cake->Dining Room]{\"the iron-key is under the table...\"\n\nYou could have sworn that's not what it read before. }How strange", false, true));
        rooms[5].addThing(new Thing(new String[]{"chairs", "chair"}, "made of the same dark wood as the table", false, true));
        rooms[5].addThing(new Thing(new String[]{"flowers", "vase", "flower"}, "very fresh — they looked as though they were picked yesterday", false, true));
        rooms[5].addThing(new Thing(new String[]{"runner", "table-runner", "linen"}, "a piece of white linen", false, true));

        //Adds items to the Kitchen
        rooms[6].addExit("south", rooms[5]);
        rooms[6].addThing(new OpenableContainer(new String[]{"refrigerator", "fridge"}, "tall and white", false, false));
        rooms[6].addThing(new OpenableContainer(new String[]{"cabinets", "cabinet"}, "made of light wood", false, true));

        rooms[6].addThing(new Thing(new String[]{"floor"}, "white tile", false, true));
        rooms[6].addThing(new Thing(new String[]{"walls", "wall"}, "pale green", false, true));
        rooms[6].addThing(new Thing(new String[]{"doorway"}, "painted white and leads back to the dining room", false, true));
        rooms[6].addThing(new Thing(new String[]{"curtains", "gingham"}, "green gingham", false, true));
        rooms[6].findThing("cabinets").addThing(new Thing(new String[]{"cup-of-flour", "cup of flour", "flour"}, "white and powdery", true, false));
        rooms[6].findThing("cabinets").addThing(new Thing(new String[]{"cup-of-sugar", "cup of sugar", "sugar"}, "granular and sweet", true, false));
        rooms[6].findThing("refrigerator").addThing(new Thing(new String[]{"egg"}, "a smooth brown sphere", true, false));
        rooms[6].findThing("cabinets").addThing(new Thing(new String[]{"peanut-butter-jar", "peanut-butter", "peanut butter", "peanut", "butter", "jar"}, "creamy and nutty", true, false));
        rooms[6].findThing("refrigerator").addThing(new Thing(new String[]{"chicken-breast", "chicken", "breast"}, "white-pink and uncooked", true, false)); 

        //Adds items to the Upstairs Hallway
        rooms[7].addExit("north", rooms[8]);
        rooms[7].addExit("south", rooms[10]);
        rooms[7].addExit("west", rooms[9]);
        rooms[7].addExit("down", rooms[1]);

        rooms[7].addThing(new Thing(new String[]{"floor"}, "nondescript", false, true));
        rooms[7].addThing(new Thing(new String[]{"walls", "wall"}, "painted the same light cream color as the entryway", false, true));
        rooms[7].addThing(new Thing(new String[]{"staircase", "stairs"}, "narrow and rickety, with a wooden bannister", false, true));
        rooms[7].addThing(new Animal(new String[]{"husky", "dog"}, "furry and gray. He's lying on a dog bed", false, 15, new String[]{"dog-biscuit"}));
        rooms[7].addThing(new Thing(new String[]{"dog-bed", "bed"}, "soft and quite large", false, true));

        //Adds items to the Blue Bedroom
        rooms[8].addExit("south", rooms[7]);

        rooms[8].addThing(new Thing(new String[]{"floor", "rug"}, "covered in a soft blue carpet", false, true));
        rooms[8].addThing(new Thing(new String[]{"walls", "wall", "wallpaper"}, "papered in an abstract blue pattern", false, true));
        rooms[8].addThing(new Thing(new String[]{"doorway"}, "painted white and leads back to the hallway", false, true));
        rooms[8].addThing(new Thing(new String[]{"bed", "quilt"}, "large and has a checkered blue quilt on it[if flashlight:!known]{. It looks like there's something under the pillow}", false, true));
        rooms[8].addThing(new Thing(new String[]{"pillow"}, "soft and white[if flashlight:!known]{. Lifting it up, you discover a flashlight}", true, true));
        rooms[8].addThing(new FueledMachine(new String[]{"flashlight"}, new String[]{"battery"}, "clunky and silver[if flashlight:on]{, but passably bright}[if flashlight:!fueled]{. It doesn't have a battery}", "The flashlight clicks on.", "The flashlight clicks off.", true, true, true));
        rooms[8].addThing(new Thing(new String[]{"painting", "landscape", "farmhouse", "hills"}, "a large oil landscape of a small farmhouse tucked between rolling hills", false, true));

        //Adds items to the Billiard Room
        rooms[9].addExit("east", rooms[7]);

        rooms[9].addThing(new Thing(new String[]{"floor"}, "made of wide, dark boards", false, true));
        rooms[9].addThing(new Thing(new String[]{"walls", "wall"}, "panelled with dark wood", false, true));
        rooms[9].addThing(new FueledMachine(new String[]{"jukebox"}, new String[]{"coin"}, "an ugly thing from the 80s[if jukebox:on]{. It's now playing an obscure 50's song that you don't know the name of}[if jukebox:!fueled]{. It looks like it takes a single coin}", "The jukebox rattles to life and starts playing Smooth Criminal by Michael Jackson.[if note:!known]{\nAs it plays, you notice a note hidden behind it.}", "The jukebox clicks off and goes dark.", false, false, false));
        rooms[9].addThing(new Thing(new String[]{"end-table", "end"}, "a small, spindly thing with a single drawer. There's a potted plant on it — a surprisingly well-cared for orchid", false, false));
        rooms[9].addThing(new Thing(new String[]{"orchid", "pot", "plant", "dirt"}, "tall and yellow, in a red pot[if hexagonal-cylinder:!known]{. There seems to be a hexagonal cylinder half-buried in the dirt}", false, true));
        rooms[9].addThing(new Thing(new String[]{"hexagonal-cylinder", "cylinder"}, "encrusted in dirt", true, true));
        rooms[9].addThing(new OpenableContainer(new String[]{"drawer"}, "very small and half-hidden under the tabletop", false, true));
        rooms[9].addThing(new Thing(new String[]{"pool-table", "pool", "table", "velvet"}, "covered in green velvet. For some reason, [if pool-ball:!known]{there's only one ball and}[if pool-ball:known]{there are} no cues", false, false));
        rooms[9].addThing(new Thing(new String[]{"pool-ball", "ball"}, "a red ball with the number 7 on it", true, true));
        rooms[9].addThing(new Thing(new String[]{"note"}, "faded and crumpled. It reads: \"Lucy, when you see this, I've hidden the key to the space under the library in the-\" The writing abrubtly cuts off", true, true));

        rooms[9].findThing("drawer").addThing(new Thing(new String[]{"coin"}, "a shiny silver quarter", true, false));

        //Adds items to the Yellow Bedroom
        rooms[10].addExit("north", rooms[7]);

        rooms[10].addThing(new Thing(new String[]{"floor"}, "a light tan color that clashes badly with the walls", false, true));
        rooms[10].addThing(new Thing(new String[]{"walls", "wall", "wallpaper"}, "very bright", false, true));
        rooms[10].addThing(new Thing(new String[]{"bed", "bedspread"}, "rickety, with a faded peach-colored bedspread", false, true));
        rooms[10].addThing(new Thing(new String[]{"bedside-table", "table"}, "as rickety as the bed. It has a drawer tucked beneath it", false, false));
        rooms[10].addThing(new OpenableContainer(new String[]{"drawer"}, "part of the bedside-table", false, true));
        
        rooms[10].findThing("drawer").addThing(new Thing(new String[]{"battery"}, "1.5-volt", true, false));
        rooms[10].findThing("drawer").addThing(new Thing(new String[]{"chocolate-bar", "chocolate", "bar"}, "earthy and rich", true, false));

        //Returns the array of Rooms
        return rooms;
    }
}